# -----------------------------------------------------------------------------
# $Id: Rehash.pm,v 1.1 2004/08/22 11:28:44 topia Exp $
# -----------------------------------------------------------------------------
# copyright (C) 2004 Topia <topia@clovery.jp>. all rights reserved.
package Client::Rehash;
use strict;
use warnings;
use base qw(Module);
use Mask;
use Multicast;
use NumericReply;
use Timer;

my $timer_name = __PACKAGE__.'/timer';

sub new {
    my $class = shift;
    my $this = $class->SUPER::new(@_);
}

sub destruct {
    my ($this) = shift;

    # timer в
    foreach my $client (RunLoop->shared_loop->clients_list) {
	my $timer = $client->remark($timer_name);
	if (defined $timer) {
	    $client->remark($timer_name, undef, 'delete');
	    $timer->uninstall;
	}
    }
}

sub message_arrived {
    my ($this,$msg,$sender) = @_;

    # 饤ȤΥå
    if ($sender->isa('IrcIO::Client')) {
	my $runloop = RunLoop->shared_loop;
	# ꤵ줿ޥɤ?
	if (Mask::match_deep([$this->config->command_nick('all')], $msg->command)) {
	    if (!defined $msg->param(0)) {
	    } elsif ($msg->param(0) eq $runloop->current_nick) {
	    } else {
		$sender->send_message(
		    IRCMessage->new(
			Prefix => $msg->param(0).'!'.$sender->username.'@'.
			    $sender->client_host,
			Command => 'NICK',
			Param => $runloop->current_nick,
		       ));

	    }
	    # Ǿä
	    return undef;
	} elsif (Mask::match_deep([$this->config->command_names('all')], $msg->command)) {
	    my @channels = map {
		my $network_name = $_->network_name;
		map {
		    [$network_name, $_->name];
		} $_->channels_list;
	    } $runloop->networks_list;
	    $sender->remark($timer_name, Timer->new(
	       Interval => (defined $this->config->interval ?
				$this->config->interval : 2),
	       Repeat => 1,
	       Code => sub {
		   my $timer = shift;
		   my $runloop = RunLoop->shared_loop;
		   while (1) {
		       my $entry = shift(@channels);
		       if (defined $entry && $sender->connected) {
			   my ($network_name, $ch_name) = @$entry;
			   my $network = $runloop->network($network_name);
			   my $flush_namreply = sub {
			       my $msg = shift;
			       $msg->param(0, $runloop->current_nick);
			       $sender->send_message($msg);
			   };
			   if (!defined $network) {
			       # network disconnected. ignore
			       next;
			   }
			   my $ch = $network->channel($ch_name);
			   if (!defined $ch) {
			       # parted channel; ignore
			       next;
			   }
			   $sender->do_namreply($ch, $network,
						undef, $flush_namreply);
		       } else {
			   $sender->remark($timer_name, undef, 'delete');
			   $timer->uninstall;
		       }
		       last;
		   }
	       },
	      )->install);

	    # Ǿä
	    return undef;
	}
    }

    return $msg;
}

1;
=pod
info: ͥʬ names å򥯥饤Ȥ롣
default: off

# ȤȤϥ饤ȤκƽŪ˺äΤǤ names Ƥ
# ʤ饤Ȥ¿Τǡ multi-server-mode  Tiarra 
# ˤ Tiarra ĤʤǤ͸ˤޤ

# names ǥ˥åꥹȤ򹹿Ƥ륯饤:
#   Tiarra
# Ƥʤ饤: (ϳǧСޤ)
#   LimeChat(1.18)

# nick rehash ˻Ȥޥɤꤷޤ
# ѥ᡼ȤƸߥ饤ȤǧƤ nick ꤷƤ
command-nick: rehash-nick

# names rehash ˻Ȥޥɤꤷޤ
command-names: rehash-names

# ͥȥͥδ֤ΥȤꤷޤ
interval: 2
=cut
